# -*- coding: utf-8 -*-
"""
app.inventory.forms
~~~~~~~~~~~~~~~~~~~

This module defines the inventory blueprint forms.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
import re
from flask_wtf import FlaskForm
from wtforms import SelectField, StringField, validators
from .. import utils, models

MAC_ADDRESS_RE = re.compile('^(?:[0-9a-fA-F]{2}[:-]){5}[0-9a-fA-F]{2}$')


def check_mac_addresses_list(form, field):
    for address in field.data.split():
        if MAC_ADDRESS_RE.match(address) is None:
            raise validators.ValidationError('Invalid Mac address')


class AttributeForm(FlaskForm):
    name = StringField('name', validators=[validators.DataRequired()])


class ItemForm(FlaskForm):
    ics_id = StringField('ICS id',
                         validators=[validators.InputRequired(),
                                     validators.Regexp(models.ICS_ID_RE)])
    serial_number = StringField('Serial number',
                                validators=[validators.InputRequired()])
    manufacturer_id = SelectField('Manufacturer')
    model_id = SelectField('Model')
    location_id = SelectField('Location')
    status_id = SelectField('Status')
    parent_id = SelectField('Parent')
    mac_addresses = StringField(
        'MAC addresses',
        description='space separated list of MAC addresses',
        validators=[validators.Optional(), check_mac_addresses_list])

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.manufacturer_id.choices = utils.get_model_choices(models.Manufacturer, allow_blank=True)
        self.model_id.choices = utils.get_model_choices(models.Model, allow_blank=True)
        self.location_id.choices = utils.get_model_choices(models.Location, allow_blank=True)
        self.status_id.choices = utils.get_model_choices(models.Status, allow_blank=True)
        self.parent_id.choices = utils.get_model_choices(models.Item, allow_blank=True, attr='ics_id')
