# -*- coding: utf-8 -*-
"""
app.networks.forms
~~~~~~~~~~~~~~~~~~

This module defines the networks blueprint forms.

:copyright: (c) 2017 European Spallation Source ERIC
:license: BSD 2-Clause, see LICENSE for more details.

"""
from flask_wtf import FlaskForm
from wtforms import SelectField, StringField, validators
from .. import utils, models


class NoValidateSelectField(SelectField):
    """SelectField with no choices validation

    By default a SelectField tries to validate the selected value
    against the list of choices. This is not possible when the choices
    are dynamically created on the browser side.
    """

    def pre_validate(self, form):
        pass


class HostForm(FlaskForm):
    network_id = SelectField('Network')
    # The list of IPs is dynamically created on the browser side
    # depending on the selected network
    ip = NoValidateSelectField('IP', choices=[])
    name = StringField('Hostname',
                       description='hostname must be 2-20 characters long and contain only letters, numbers and dash',
                       validators=[validators.InputRequired(),
                                   validators.Regexp(models.HOST_NAME_RE)],
                       filters=[utils.lowercase_field])
    description = StringField('Description')
    mac_id = SelectField('MAC')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.network_id.choices = [(str(network.id), network.vlan_name) for network in models.Network.query.all()]
        self.mac_id.choices = [('', '')]
        self.mac_id.choices.extend([(str(mac.id), mac.address) for mac in models.Mac.query.all()])
